@@ -40,7 +40,6 @@ except ImportError:
 import traceback
 
 from oslo_config import cfg
-from oslo_utils import encodeutils
 from oslo_utils import importutils
 import six
 from six import moves
@@ -92,21 +91,6 @@ class BaseLoggerAdapter(logging.LoggerAdapter):
         self.log(TRACE, msg, *args, **kwargs)
 
 
-def _ensure_unicode(msg):
-    """Do our best to turn the input argument into a unicode object.
-    """
-    if not isinstance(msg, six.text_type):
-        if isinstance(msg, six.binary_type):
-            msg = encodeutils.safe_decode(
-                msg,
-                incoming='utf-8',
-                errors='xmlcharrefreplace',
-            )
-        else:
-            msg = six.text_type(msg)
-    return msg
-
-
 class KeywordArgumentAdapter(BaseLoggerAdapter):
     """Logger adapter to add keyword arguments to log record's extra data
 
@@ -126,7 +110,6 @@ class KeywordArgumentAdapter(BaseLoggerAdapter):
     """
 
     def process(self, msg, kwargs):
-        msg = _ensure_unicode(msg)
         # Make a new extra dictionary combining the values we were
         # given when we were constructed and anything from kwargs.
         extra = {}
@@ -219,6 +202,15 @@ def _load_log_config(log_config_append):
         raise LogConfigError(log_config_append, six.text_type(exc))
 
 
+def _mutate_hook(conf, fresh):
+    """Reconfigures oslo.log according to the mutated options."""
+
+    # verbose is deprecated, so I didn't make it mutable, so there's no need to
+    # test for it here.
+    if (None, 'debug') in fresh:
+        _refresh_root_level(conf.debug, conf.verbose)
+
+
 def register_options(conf):
     """Register the command line and configuration options used by oslo.log."""
 
@@ -234,6 +226,8 @@ def register_options(conf):
     conf.register_opts(_options.log_opts)
     formatters._store_global_conf(conf)
 
+    conf.register_mutate_hook(_mutate_hook)
+
 
 def setup(conf, product_name, version='unknown'):
     """Setup logging for the current application."""
@@ -301,6 +295,27 @@ def _find_facility(facility):
     return getattr(syslog, facility)
 
 
+def _refresh_root_level(debug, verbose):
+    """Set the level of the root logger.
+
+    If 'debug' is True, the level will be DEBUG. Otherwise we look at 'verbose'
+    - if that is True, the level will be INFO. If neither are set, the level
+    will be WARNING.
+
+    Note the 'verbose' option is deprecated.
+
+    :param debug
+    :param verbose
+    """
+    log_root = getLogger(None).logger
+    if debug:
+        log_root.setLevel(logging.DEBUG)
+    elif verbose:
+        log_root.setLevel(logging.INFO)
+    else:
+        log_root.setLevel(logging.WARNING)
+
+
 def _setup_logging_from_conf(conf, project, version):
     log_root = getLogger(None).logger
 
@@ -349,13 +364,7 @@ def _setup_logging_from_conf(conf, project, version):
                                                          version=version,
                                                          datefmt=datefmt,
                                                          config=conf))
-
-    if conf.debug:
-        log_root.setLevel(logging.DEBUG)
-    elif conf.verbose:
-        log_root.setLevel(logging.INFO)
-    else:
-        log_root.setLevel(logging.WARNING)
+    _refresh_root_level(conf.debug, conf.verbose)
 
     for pair in conf.default_log_levels:
         mod, _sep, level_name = pair.partition('=')
